@@ -25,11 +25,14 @@ import pytest
 from airflow.exceptions import AirflowException, XComNotFound
 from airflow.models.taskinstance import TaskInstance
 from airflow.models.taskmap import TaskMap
-from airflow.utils import timezone
 from airflow.utils.task_instance_session import set_current_task_instance_session
-from airflow.utils.trigger_rule import TriggerRule
 
-from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_1, AIRFLOW_V_3_0_PLUS, XCOM_RETURN_KEY
+from tests_common.test_utils.version_compat import (
+    AIRFLOW_V_3_0_1,
+    AIRFLOW_V_3_0_PLUS,
+    AIRFLOW_V_3_1_PLUS,
+    XCOM_RETURN_KEY,
+)
 from unit.standard.operators.test_python import BasePythonTest
 
 if AIRFLOW_V_3_0_PLUS:
@@ -37,7 +40,6 @@ if AIRFLOW_V_3_0_PLUS:
     from airflow.sdk.bases.decorator import DecoratedMappedOperator
     from airflow.sdk.definitions._internal.expandinput import DictOfListsExpandInput
     from airflow.sdk.definitions.mappedoperator import MappedOperator
-
 else:
     from airflow.decorators import (  # type: ignore[attr-defined,no-redef]
         setup,
@@ -46,12 +48,18 @@ else:
     )
     from airflow.decorators.base import DecoratedMappedOperator  # type: ignore[no-redef]
     from airflow.models.baseoperator import BaseOperator  # type: ignore[no-redef]
-    from airflow.models.dag import DAG  # type: ignore[assignment]
+    from airflow.models.dag import DAG  # type: ignore[assignment,no-redef]
     from airflow.models.expandinput import DictOfListsExpandInput
-    from airflow.models.mappedoperator import MappedOperator
+    from airflow.models.mappedoperator import MappedOperator  # type: ignore[assignment,no-redef]
     from airflow.models.xcom_arg import XComArg
     from airflow.utils.task_group import TaskGroup  # type: ignore[no-redef]
 
+if AIRFLOW_V_3_1_PLUS:
+    from airflow.sdk import TriggerRule, timezone
+else:
+    from airflow.utils import timezone  # type: ignore[attr-defined,no-redef]
+    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]
+
 pytestmark = pytest.mark.db_test
 
 
@@ -672,13 +680,11 @@ def test_mapped_decorator_shadow_context() -> None:
     def print_info(message: str, run_id: str = "") -> None:
         print(f"{run_id}: {message}")
 
-    with pytest.raises(ValueError) as ctx:
+    with pytest.raises(ValueError, match=r"cannot call partial\(\) on task context variable 'run_id'"):
         print_info.partial(run_id="hi")
-    assert str(ctx.value) == "cannot call partial() on task context variable 'run_id'"
 
-    with pytest.raises(ValueError) as ctx:
+    with pytest.raises(ValueError, match=r"cannot call expand\(\) on task context variable 'run_id'"):
         print_info.expand(run_id=["hi", "there"])
-    assert str(ctx.value) == "cannot call expand() on task context variable 'run_id'"
 
 
 def test_mapped_decorator_wrong_argument() -> None:
@@ -694,9 +700,10 @@ def test_mapped_decorator_wrong_argument() -> None:
         print_info.expand(wrong_name=["hi", "there"])
     assert str(ct.value) == "expand() got an unexpected keyword argument 'wrong_name'"
 
-    with pytest.raises(ValueError) as cv:
+    with pytest.raises(
+        ValueError, match=r"expand\(\) got an unexpected type 'str' for keyword argument 'message'"
+    ):
         print_info.expand(message="hi")
-    assert str(cv.value) == "expand() got an unexpected type 'str' for keyword argument 'message'"
 
 
 def test_mapped_decorator():
@@ -777,7 +784,7 @@ def test_partial_mapped_decorator() -> None:
 
 
 def test_mapped_decorator_unmap_merge_op_kwargs(dag_maker, session):
-    with dag_maker(session=session):
+    with dag_maker(session=session, serialized=True):
 
         @task_decorator
         def task1():
@@ -788,18 +795,22 @@ def test_mapped_decorator_unmap_merge_op_kwargs(dag_maker, session):
 
         task2.partial(arg1=1).expand(arg2=task1())
 
-    run = dag_maker.create_dagrun()
+    run = dag_maker.create_dagrun(session=session)
 
     # Run task1.
     dec = run.task_instance_scheduling_decisions(session=session)
     assert [ti.task_id for ti in dec.schedulable_tis] == ["task1"]
-    dec.schedulable_tis[0].run(session=session)
+    ti = dec.schedulable_tis[0]
+    dag_maker.run_ti(task_id=ti.task_id, dag_run=run, session=session)
 
     # Expand task2.
     dec = run.task_instance_scheduling_decisions(session=session)
     assert [ti.task_id for ti in dec.schedulable_tis] == ["task2"]
     ti = dec.schedulable_tis[0]
 
+    # Use the real task for unmapping to mimic actual execution path
+    ti.task = dag_maker.dag.task_dict[ti.task_id]
+
     if AIRFLOW_V_3_0_PLUS:
         unmapped = ti.task.unmap((ti.get_template_context(session),))
     else:
@@ -879,11 +890,16 @@ def test_task_decorator_has_doc_attr():
 
 def test_upstream_exception_produces_none_xcom(dag_maker, session):
     from airflow.exceptions import AirflowSkipException
-    from airflow.utils.trigger_rule import TriggerRule
+
+    try:
+        from airflow.sdk import TriggerRule
+    except ImportError:
+        # Compatibility for Airflow < 3.1
+        from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]
 
     result = None
 
-    with dag_maker(session=session) as dag:
+    with dag_maker(session=session, serialized=True) as dag:
 
         @dag.task()
         def up1() -> str:
@@ -905,22 +921,27 @@ def test_upstream_exception_produces_none_xcom(dag_maker, session):
     decision = dr.task_instance_scheduling_decisions(session=session)
     assert len(decision.schedulable_tis) == 2  # "up1" and "up2"
     for ti in decision.schedulable_tis:
-        ti.run(session=session)
+        dag_maker.run_ti(ti.task_id, dag_run=dr, session=session)
 
     decision = dr.task_instance_scheduling_decisions(session=session)
     assert len(decision.schedulable_tis) == 1  # "down"
-    decision.schedulable_tis[0].run(session=session)
+    dag_maker.run_ti(decision.schedulable_tis[0].task_id, dag_run=dr, session=session)
     assert result == "'example' None"
 
 
 @pytest.mark.parametrize("multiple_outputs", [True, False])
 def test_multiple_outputs_produces_none_xcom_when_task_is_skipped(dag_maker, session, multiple_outputs):
     from airflow.exceptions import AirflowSkipException
-    from airflow.utils.trigger_rule import TriggerRule
+
+    try:
+        from airflow.sdk import TriggerRule
+    except ImportError:
+        # Compatibility for Airflow < 3.1
+        from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]
 
     result = None
 
-    with dag_maker(session=session) as dag:
+    with dag_maker(session=session, serialized=True) as dag:
 
         @dag.task()
         def up1() -> str:
@@ -944,16 +965,16 @@ def test_multiple_outputs_produces_none_xcom_when_task_is_skipped(dag_maker, ses
     decision = dr.task_instance_scheduling_decisions(session=session)
     assert len(decision.schedulable_tis) == 2  # "up1" and "up2"
     for ti in decision.schedulable_tis:
-        ti.run(session=session)
+        dag_maker.run_ti(ti.task_id, dag_run=dr, session=session)
 
     decision = dr.task_instance_scheduling_decisions(session=session)
     assert len(decision.schedulable_tis) == 1  # "down"
     if multiple_outputs:
-        decision.schedulable_tis[0].run(session=session)
+        dag_maker.run_ti(decision.schedulable_tis[0].task_id, dag_run=dr, session=session)
         assert result == "'example' None"
     else:
         with pytest.raises(XComNotFound):
-            decision.schedulable_tis[0].run(session=session)
+            dag_maker.run_ti(decision.schedulable_tis[0].task_id, dag_run=dr, session=session)
 
 
 @pytest.mark.filterwarnings("error")
@@ -990,7 +1011,7 @@ def test_task_decorator_asset(dag_maker, session):
     if AIRFLOW_V_3_0_PLUS:
         session.add(AssetActive.for_asset(asset))
 
-    with dag_maker(session=session) as dag:
+    with dag_maker(session=session, serialized=True) as dag:
 
         @dag.task()
         def up1() -> Asset:
@@ -1012,15 +1033,18 @@ def test_task_decorator_asset(dag_maker, session):
     dr: DagRun = dag_maker.create_dagrun()
     decision = dr.task_instance_scheduling_decisions(session=session)
     assert len(decision.schedulable_tis) == 1  # "up1"
-    decision.schedulable_tis[0].run(session=session)
+    ti = decision.schedulable_tis[0]
+    dag_maker.run_ti(ti.task_id, dag_run=dr, session=session)
 
     decision = dr.task_instance_scheduling_decisions(session=session)
     assert len(decision.schedulable_tis) == 1  # "up2"
-    decision.schedulable_tis[0].run(session=session)
+    ti = decision.schedulable_tis[0]
+    dag_maker.run_ti(ti.task_id, dag_run=dr, session=session)
 
     decision = dr.task_instance_scheduling_decisions(session=session)
     assert len(decision.schedulable_tis) == 1  # "down"
-    decision.schedulable_tis[0].run(session=session)
+    ti = decision.schedulable_tis[0]
+    dag_maker.run_ti(ti.task_id, dag_run=dr, session=session)
     assert result == uri
 
 
