@@ -27,7 +27,7 @@ import pytest
 
 from airflow import settings
 from airflow.exceptions import AirflowException, AirflowSensorTimeout, AirflowSkipException, TaskDeferred
-from airflow.models import DagBag, DagRun, TaskInstance
+from airflow.models import DagRun, TaskInstance
 from airflow.models.dag import DAG
 from airflow.models.serialized_dag import SerializedDagModel
 from airflow.models.xcom_arg import XComArg
@@ -56,7 +56,7 @@ from airflow.utils.types import DagRunType
 from tests_common.test_utils.dag import create_scheduler_dag, sync_dag_to_db, sync_dags_to_db
 from tests_common.test_utils.db import clear_db_runs
 from tests_common.test_utils.mock_operators import MockOperator
-from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS, AIRFLOW_V_3_1_PLUS
+from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS, AIRFLOW_V_3_1_PLUS, AIRFLOW_V_3_2_PLUS
 
 if AIRFLOW_V_3_0_PLUS:
     from airflow.models.dag_version import DagVersion
@@ -73,6 +73,12 @@ else:
     from airflow.utils.task_group import TaskGroup  # type: ignore[no-redef]
     from airflow.utils.timezone import coerce_datetime, datetime  # type: ignore[attr-defined,no-redef]
 
+if AIRFLOW_V_3_2_PLUS:
+    from airflow.dag_processing.dagbag import DagBag
+else:
+    from airflow.models.dagbag import DagBag  # type: ignore[attr-defined, no-redef]
+
+
 pytestmark = pytest.mark.db_test
 
 TI = TaskInstance
@@ -86,6 +92,8 @@ DEV_NULL = "/dev/null"
 TASK_ID = "external_task_sensor_check"
 EXTERNAL_DAG_ID = "child_dag"  # DAG the external task sensor is waiting on
 EXTERNAL_TASK_ID = "child_task"  # Task the external task sensor is waiting on
+EXTERNAL_ID_AND_IDS_PROVIDE_ERROR = "Only one of `external_task_id` or `external_task_ids` may be provided to ExternalTaskSensor; use external_task_id or external_task_ids or external_task_group_id."
+EXTERNAL_IDS_AND_TASK_GROUP_ID_PROVIDE_ERROR = "Only one of `external_task_group_id` or `external_task_ids` may be provided to ExternalTaskSensor; use external_task_id or external_task_ids or external_task_group_id."
 
 
 @pytest.fixture(autouse=True)
@@ -203,7 +211,10 @@ class TestExternalTaskSensorV2:
         op.run(start_date=DEFAULT_DATE, end_date=DEFAULT_DATE, ignore_ti_state=True)
 
     def test_raise_with_external_task_sensor_task_id_and_task_ids(self):
-        with pytest.raises(ValueError) as ctx:
+        with pytest.raises(
+            ValueError,
+            match=EXTERNAL_ID_AND_IDS_PROVIDE_ERROR,
+        ):
             ExternalTaskSensor(
                 task_id="test_external_task_sensor_task_id_with_task_ids_failed_status",
                 external_dag_id=TEST_DAG_ID,
@@ -211,14 +222,9 @@ class TestExternalTaskSensorV2:
                 external_task_ids=TEST_TASK_ID,
                 dag=self.dag,
             )
-        assert (
-            str(ctx.value) == "Only one of `external_task_id` or `external_task_ids` may "
-            "be provided to ExternalTaskSensor; "
-            "use external_task_id or external_task_ids or external_task_group_id."
-        )
 
     def test_raise_with_external_task_sensor_task_group_and_task_id(self):
-        with pytest.raises(ValueError) as ctx:
+        with pytest.raises(ValueError, match=EXTERNAL_IDS_AND_TASK_GROUP_ID_PROVIDE_ERROR):
             ExternalTaskSensor(
                 task_id="test_external_task_sensor_task_group_with_task_id_failed_status",
                 external_dag_id=TEST_DAG_ID,
@@ -226,14 +232,9 @@ class TestExternalTaskSensorV2:
                 external_task_group_id=TEST_TASK_GROUP_ID,
                 dag=self.dag,
             )
-        assert (
-            str(ctx.value) == "Only one of `external_task_group_id` or `external_task_ids` may "
-            "be provided to ExternalTaskSensor; "
-            "use external_task_id or external_task_ids or external_task_group_id."
-        )
 
     def test_raise_with_external_task_sensor_task_group_and_task_ids(self):
-        with pytest.raises(ValueError) as ctx:
+        with pytest.raises(ValueError, match=EXTERNAL_IDS_AND_TASK_GROUP_ID_PROVIDE_ERROR):
             ExternalTaskSensor(
                 task_id="test_external_task_sensor_task_group_with_task_ids_failed_status",
                 external_dag_id=TEST_DAG_ID,
@@ -241,11 +242,6 @@ class TestExternalTaskSensorV2:
                 external_task_group_id=TEST_TASK_GROUP_ID,
                 dag=self.dag,
             )
-        assert (
-            str(ctx.value) == "Only one of `external_task_group_id` or `external_task_ids` may "
-            "be provided to ExternalTaskSensor; "
-            "use external_task_id or external_task_ids or external_task_group_id."
-        )
 
     # by default i.e. check_existence=False, if task_group doesn't exist, the sensor will run till timeout,
     # this behaviour is similar to external_task_id doesn't exists
@@ -304,7 +300,10 @@ class TestExternalTaskSensorV2:
             )
 
     def test_external_task_sensor_wrong_failed_states(self):
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match="Valid values for `allowed_states`, `skipped_states` and `failed_states` when `external_task_id` or `external_task_ids` or `external_task_group_id` is not `None`",
+        ):
             ExternalTaskSensor(
                 task_id="test_external_task_sensor_check",
                 external_dag_id=TEST_DAG_ID,
@@ -666,7 +665,10 @@ exit 0
     def test_external_task_sensor_error_delta_and_fn(self):
         self.add_time_sensor()
         # Test that providing execution_delta and a function raises an error
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match="Only one of `execution_delta` or `execution_date_fn` may be provided to ExternalTaskSensor; not both.",
+        ):
             ExternalTaskSensor(
                 task_id="test_external_task_sensor_check_delta",
                 external_dag_id=TEST_DAG_ID,
@@ -680,7 +682,10 @@ exit 0
     def test_external_task_sensor_error_task_id_and_task_ids(self):
         self.add_time_sensor()
         # Test that providing execution_delta and a function raises an error
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match=EXTERNAL_ID_AND_IDS_PROVIDE_ERROR,
+        ):
             ExternalTaskSensor(
                 task_id="test_external_task_sensor_task_id_and_task_ids",
                 external_dag_id=TEST_DAG_ID,
@@ -711,7 +716,7 @@ exit 0
             allowed_states=["success"],
             dag=self.dag,
         )
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Duplicate task_ids"):
             op1.run(start_date=DEFAULT_DATE, end_date=DEFAULT_DATE)
 
     def test_catch_duplicate_task_ids_with_xcom_arg(self):
@@ -731,7 +736,7 @@ exit 0
             dag=self.dag,
         )
         op1.run(start_date=DEFAULT_DATE, end_date=DEFAULT_DATE)
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Duplicate task_ids"):
             op2.run(start_date=DEFAULT_DATE, end_date=DEFAULT_DATE)
 
     def test_catch_duplicate_task_ids_with_multiple_xcom_args(self):
@@ -752,11 +757,14 @@ exit 0
             dag=self.dag,
         )
         op1.run(start_date=DEFAULT_DATE, end_date=DEFAULT_DATE)
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Duplicate task_ids"):
             op2.run(start_date=DEFAULT_DATE, end_date=DEFAULT_DATE)
 
     def test_catch_invalid_allowed_states(self):
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match="Valid values for `allowed_states`, `skipped_states` and `failed_states`",
+        ):
             ExternalTaskSensor(
                 task_id="test_external_task_sensor_check_1",
                 external_dag_id=TEST_DAG_ID,
@@ -765,7 +773,10 @@ exit 0
                 dag=self.dag,
             )
 
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match="Valid values for `allowed_states`, `skipped_states` and `failed_states`",
+        ):
             ExternalTaskSensor(
                 task_id="test_external_task_sensor_check_2",
                 external_dag_id=TEST_DAG_ID,
@@ -1185,7 +1196,10 @@ class TestExternalTaskSensorV3:
 
     def test_external_task_sensor_invalid_combination(self, dag_maker):
         """Test that the sensor raises an error with invalid parameter combinations."""
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match=EXTERNAL_ID_AND_IDS_PROVIDE_ERROR,
+        ):
             with dag_maker("test_external_task_sensor_invalid_combination"):
                 ExternalTaskSensor(
                     task_id="test_external_task_sensor_check",
@@ -1195,7 +1209,10 @@ class TestExternalTaskSensorV3:
                 )
 
     def test_external_task_sensor_invalid_state(self, dag_maker):
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match="Valid values for `allowed_states`, `skipped_states` and `failed_states` when `external_task_id` or `external_task_ids` or `external_task_group_id` is not `None",
+        ):
             with dag_maker("test_external_task_sensor_invalid_state"):
                 ExternalTaskSensor(
                     task_id="test_external_task_sensor_check",
@@ -1917,7 +1934,8 @@ def dag_bag_cyclic():
 
         for dag in dags:
             if AIRFLOW_V_3_0_PLUS:
-                dag_bag.bag_dag(dag=sync_dag_to_db(dag))
+                sync_dag_to_db(dag)
+                dag_bag.bag_dag(dag=dag)
             else:
                 dag_bag.bag_dag(dag=dag, root_dag=dag)  # type: ignore[call-arg]
 
